<?php defined('KERNEL_INCLUDED') or die('kernel should be included first');
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder

//-----------------------------------------------------------------
/// @file
/// @brief #debug: error reporting, debugging, tracing
//-----------------------------------------------------------------

/// Maximum count of reported errors (see #debug class)
defineDefault('DEBUG_maxReportError', 10);


//-----------------------------------------------------------------
/// @brief Static class offering debugging tools:
/// error reporting, debugging, tracing.
//-----------------------------------------------------------------

class debug
{
	const dumpMaxRecurseDepth = 3;   ///< @see #dumpSimple(), #dumpRec(), #dump() and #dumpNow()
	const dumpIndent          = 3;   ///< Indent level when dumping imbricated values, in characters
	
	const nbLineAround        = 3;   ///< When displaying the source file lines of an error
	
	const maxReportError = DEBUG_maxReportError;   ///< Maximum count of reported errors
	
	
	public static $bCanLog = true;   ///< Allows to disable logging even if #sLogFile is defined.
	
	
	/// Initialize the debugging system.
	public static function setup()
	{
		error_reporting(~0);
		set_error_handler(array(__CLASS__, 'errorHandler'));
	}
	
	
	/// Display a fatal error page and quit.
	public static function fatalError($sMessage, $sCode = '')
	{
		$sBody = html::par('Fatal error: ' . $sMessage);
		if (isSome($sCode))
			$sBody .= html::pre(html::text($sCode));
		
		$aStackTrace = self::getStackTrace();
		$sBody .= self::formatStackTrace($aStackTrace);
		
		echo
			sDocType .
			html::tag('html',
				html::tag('head', html::tag('title', 'Fatal error')) .
				html::tag('body', $sBody));
		die;
	}
	
	
	/// @brief Hash a value and add it to the \p $aAlready array.
	///
	/// @param $value value to hash.
	/// @param[in,out] $aAlready will contain a key corresponding to the hash of the value.
	/// @return \c true if \p $aAlready did not already contained the hash of the value,
	/// \c false otherwise.
	private static function hashValue($value, array &$aAlready)
	{
		$sHash = md5(serialize($value));
		if (array_key_exists($sHash, $aAlready))
			return false;
		$aAlready[$sHash] = true;
		return true;
	}
	
	
	/// @brief Convert a value to a single line human readable string.
	/// Recurse with limited depth.
	///
	/// @param $value the value to dump.
	/// @param $iDepth the current recursivity depth, zero for root.
	/// @param $aAlready the array whose keys are the MD5 of the serialization.
	/// of the parent elements, according to recursive calls.
	/// Used to avoid printing an element twice, and to avoid infinite looping if depth is not limited.
	public static function dumpSimple($value, $iDepth = 0, array $aAlready = array())
	{
		if (is_bool($value))
			return html::i(($value) ? 'true' : 'false');
		if (is_string($value))
			return '\'' . html::text(addcslashes($value, "\0..\37")) . '\'';
		if (is_resource($value))
			return html::i('resource');
		if (is_null($value))
			return html::i('NULL');
	
		if (is_array($value) || is_object($value)) {
			$s = '';
			if (!self::hashValue($value, $aAlready))
				return html::i('recurse');
	
			$bRecurse = ($iDepth < self::dumpMaxRecurseDepth);
			$iDepth++;
	
			if (is_array($value)) {
				if ($bRecurse) {
					$a = array();
					foreach ($value as $key => $val)
						$a[] = self::dumpSimple($key, $iDepth, $aAlready) . ' &rarr; ' .
							self::dumpSimple($val, $iDepth, $aAlready);
					$s = implode(', ', $a);
				}else
					$s = 'size = ' . count($value);
				return 'array(' . $s . ')';
	
			}else{
				if ($bRecurse) {
					$a = array();
					foreach ($value as $key => $val)
						$a[] = $key . ' = ' . self::dumpSimple($val, $iDepth, $aAlready);
					if ($a)
						$s .= ': ' . implode(', ', $a);
				}
				return 'object(' . get_class($value) . $s . ')';
			}
		}
		
		@settype($value, 'string');
		return html::text($value);
	}
	
	
	/// @brief Convert a value to a human readable string, recursively.
	/// Depth is not limited.
	///
	/// @param $value the value to dump.
	/// @param $iDepth the current recursivity depth, zero for root.
	/// @param $aAlready the array whose keys are the MD5 of the serialization.
	/// of the parent elements, according to recursive calls.
	/// Used to avoid printing an element twice, and to avoid infinite looping if depth is not limited.
	public static function dumpRec($value, $iDepth = 0, array $aAlready = array())
	{
		if (is_array($value) || is_object($value)) {
			if (!self::hashValue($value, $aAlready))
				return html::i('recurse');
			
			if ($iDepth < self::dumpMaxRecurseDepth) {
				$sIndent = str_repeat(' ', $iDepth * self::dumpIndent);
				$sIndentNew = $sIndent . str_repeat(' ', self::dumpIndent);
				$iDepth++;
				
				if (is_array($value)) {
					$a[] = 'array(';
					foreach ($value as $key => $val)
						$a[] = $sIndentNew . self::dumpRec($key, $iDepth, $aAlready) .
							' &rarr; ' . self::dumpRec($val, $iDepth, $aAlready);
					$a[] = $sIndent . ')';
					return implode(LF, $a);
				
				}else{
					$a[] = 'object(' . get_class($value) . ')';
					foreach ($value as $key => $val)
						$a[] = $sIndentNew . '$' . $key . ' = ' .
							self::dumpRec($val, $iDepth, $aAlready);
					return implode(LF, $a);
				}
			}
		}
		
		return self::dumpSimple($value);
	}
	
	
	/// @brief Convert a value to a human readable string, recursively.
	/// Enclose it in <tt>\<pre\></tt> tag.
	public static function dump($value)
	{
		return html::pre(self::dumpRec($value));
	}
	
	
	/// Print immediately the result of self::dump().
	public static function dumpNow($value)
	{
		echo self::dump($value);
	}
	
	
	/// Extract a range of lines from an array and append a prefix to each of them.
	private static function extractLines(array $aLine, $iFirstLine, $nbLine, $sMark = ' ')
	{
		if ($iFirstLine < 0) {
			$nbLine += $iFirstLine;
			$iFirstLine = 0;
		}
		if ($nbLine <= 0)
			return array();
		
		$a = array();
		foreach (array_slice($aLine, $iFirstLine, $nbLine) as $sLine)
			$a[] = $sMark . $sLine;
		return $a;
	}
	
	
	/// @brief Return a line of a file and some lines around it.
	///
	/// @param $sFile the name of the file to read.
	/// @param $iLine the index of the line to get, 1 for the first line.
	/// @param $nbLineAround the number of lines around \p $iLine to include,
	/// upwards and downwards.
	public static function extractFileLines($sFile, $iLine, $nbLineAround = 0)
	{
		$iLine--;
		$aLine = @file($sFile);
		if (empty($aLine[$iLine]))
			return '';
		
		return array_merge(
			self::extractLines($aLine, $iLine - $nbLineAround, $nbLineAround),
			self::extractLines($aLine, $iLine, 1, ($nbLineAround > 0) ? '*' : ' '),
			self::extractLines($aLine, $iLine + 1, $nbLineAround));
	}
	
	
	/// @brief Format PHP lines extracted by extractFileLines().
	/// Each line should have a one-character prefix.
	/// A \c '*' prefix means that the line should be highlighted.
	public static function formatLines($aLine)
	{
		if (!$aLine)
			return '';
		
		$s = '';
		$sLastStyle = null;
		foreach ($aLine as $sLine) {
			$sStyle = 'margin: 0em';
			if ($sLine[0] == '*')
				$sStyle .= '; background: #FFE0E0';
			if ($sLastStyle !== $sStyle) {
				if ($s)
					$s .= '</pre>';
				$sLastStyle = $sStyle;
				$s .= "<pre style='$sStyle'>";
			}
			$s .= html::text(substr($sLine, 1));
		}
		$s .= '</pre>';
		
		return str_replace(TAB, '   ', $s);
	}
	
	
	/// Return the current stack trace, ignoring the last \c $nbLevelIgnore levels.
	public static function getStackTrace($nbLevelIgnore = 0)
	{
		$aStackTrace = array_slice(debug_backtrace(), $nbLevelIgnore + 1);
		foreach ($aStackTrace as &$trace) {
			settype($trace['file'], 'string');
			settype($trace['line'], 'integer');
			$trace['source'] = self::extractFileLines($trace['file'], $trace['line']);
		}
		return $aStackTrace;
	}
	
	
	/// Format a stack trace in XHTML.
	public static function formatStackTrace(array $aStackTrace)
	{
		$s = '';
		foreach ($aStackTrace as &$trace) {
			$sFunction = $trace['function'];
			if (isset($trace['class']))
				$sFunction = $trace['class'] . '::' . $sFunction;
			
			settype($trace['args'],   'array');
			settype($trace['source'], 'array');
			
			$sTrace = html::b(html::text($trace['file'])) . ' line ' . html::b($trace['line']);
			
			$aSource = $trace['source'];
			if (count($aSource) == 1)
				$aSource = array('*' . firstElement($aSource));
			if ($aSource)
				$sTrace .= html::div(self::formatLines($aSource), attr::style('margin: 0.5em 0em'));
			
			$sArgs = implode(',' . CRLF, array_map(array(__CLASS__, 'dumpRec'), $trace['args']));
			if ($sArgs)
				$sArgs = html::div($sArgs, attr::style('margin: 0em 0em 0em 2em;'));
			
			$sTrace .= html::pre(
				html::text($sFunction) . '(' . $sArgs . ')',
				attr::style('margin: 0em 0em 0em 2em;'));
			
			$s .= html::div($sTrace, attr::style('margin-top: 1em; ; padding-top: 0.2em; border-top: 1px solid black'));
		}
		if ($s)
			$s = html::div($s, attr::style('border-bottom: 3px solid black'));
		
		return $s;
	}
	
	
	/// Compose an error report using the given information.
	private static function formatErrorReport(stdClass $data)
	{
		$aKind = getList(
			'E_ERROR;E_WARNING;E_PARSE;E_NOTICE;E_CORE_ERROR;E_CORE_WARNING;' .
			'E_COMPILE_ERROR;E_COMPILE_WARNING;E_USER_ERROR;E_USER_WARNING;E_USER_NOTICE;E_STRICT');
		$sKindName = 'E_UNKNOWN';
		foreach ($aKind as $sKind)
			if ($data->errno == constant($sKind))
				$sKindName = $sKind;
		
		$sFile = html::text($data->errfile);
		$sSource = self::formatLines($data->aSource);
		if ($sSource != '')
			$sSource = html::div($sSource) . LF;
		$sTime = @date('Y-m-d h:i:s', $data->time) . '  ';
		
		$s = html::par($sTime . "<b>$sKindName</b> [$data->errno] $data->errstr in <b>$sFile</b> on line <b>$data->errline</b>") .
			LF . $sSource;
		$s .= self::formatStackTrace($data->aStackTrace) . LF;
		
		return $s;
	}
	
	
	/// Enhanced error handler: prints stackdump.
	public static function errorHandler($errno, $errstr, $errfile, $errline)
	{
		static $s_nbError = 0;
		
		if (!(error_reporting() & $errno) || ++$s_nbError > self::maxReportError)
			return;
		
		$aSource = self::extractFileLines($errfile, $errline, self::nbLineAround);
		$aStackTrace = self::getStackTrace(1);
		$data = (object)compact(getList('errno;errstr;errfile;errline;aSource;time;aStackTrace'));
		
		if (self::$bCanLog && !is_null(sLogFile)) {
			if (!is_file(sLogFile) || filesize(sLogFile) < maxSizeLogFile) {
				$s = serialize($data);
				$s = str_replace(LF, '$' . LF . '$', $s) . '#' . LF . '#';
				$hfLog = @fopen(sLogFile, 'a');
				@fwrite($hfLog, $s);
				@fclose($hfLog);
			}
		}
		
		if (bDebug)
			echo self::formatErrorReport($data);
	}
	
	
	/// Return the contents of the error log file formatted in XHTML.
	public static function formatLogFile()
	{
		$sLog = @file_get_contents(sLogFile);
		if (!$sLog)
			return '';
		
		$s = '';
		foreach (explode('#' . LF . '#', $sLog) as $sData) {
			$data = unserialize(str_replace('$' . LF . '$', LF, $sData));
			if ($data !== false) {
				$s .= html::hr();
				$s .= self::formatErrorReport($data);
			}
		}
		return $s . html::hr();
	}
	
	
	/// @brief Return the full URL of the displayed page.
	/// POST variables are included by default.
	/// The page is supposed to use HTTP protocol (no HTTPS).
	public static function getRequestURL($bWithPost = true)
	{
		$target = target::fromURL('http://' . $_SERVER->HTTP_HOST . $_SERVER->REQUEST_URI);
		if ($bWithPost)
			$target->addParams($_POST->getArray());
		return $target->url();
	}
}

?>