<?php defined('KERNEL_INCLUDED') or die('kernel should be included first');
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder

//-----------------------------------------------------------------
/// @file
/// @brief HTML writing classes:
/// - #html: HTML abstraction to write HTML code
/// - #form: HTML forms writing
/// - #htmlframeset: HTML frames writing
//-----------------------------------------------------------------

//-----------------------------------------------------------------
/// XHTML writing
//-----------------------------------------------------------------

class html extends xmlwriting
{
	/// @brief Convert \c ' characters to the typographic apostroph
	/// aka Right Single QUOtation mark \c &rsquo;.
	static function beautifyQuotes($s)
	{
		return str_replace('\'', '', $s);
	}
	
	/// @brief HTML-escape a Windows-1252 string
	/// and convert its \c ' characters to the typographic apostroph
	/// aka Right Single QUOtation mark \c &rsquo;.
	/// @see text, beautifyQuotes
	static function textQuotes($s)
	{
		return self::text(self::beautifyQuotes($s));
	}
	
	
	/// @name Text tags writing
	/// Most functions take an \p $aAttr argument: the attributes associative array.
	/// Most of the tag generating functions take a \p $sContents argument:
	/// the string to be enclosed by the tag.
	/// @{
	
	/// Generate a line breaking tag.
	static function br()
	{
		return '<br />' . sXMLNewline;
	}
	
	/// Generate an unbreakable space entity.
	static function nbsp()
	{
		return '&nbsp;';
	}
	
	/// @brief Convert a list of strings to a single line breaking tag separated string.
	///
	/// If the first argument is an array, process the strings of the array.
	/// If not, process all arguments (there may be more than one).
	static function brize($a)
	{
		if (!is_array($a))
			$a = func_get_args();
		return implode(self::br(), $a);
	}
	
	static function hr(array $aAttr = array()) { return html::anchor('hr', $aAttr); }   ///< Generate an <tt>\<hr\></tt> tag.
	
	static function par   ($sContents, array $aAttr = array()) { return html::tag('p',      $sContents, $aAttr, true, true); } ///< Generate a <tt>\<p\></tt> tag.
	static function div   ($sContents, array $aAttr = array()) { return html::tag('div',    $sContents, $aAttr); } ///< Generate a <tt>\<div\></tt> tag.
	static function pre   ($sContents, array $aAttr = array()) { return html::tag('pre',    $sContents, $aAttr); } ///< Generate a <tt>\<pre\></tt> tag.
	static function span  ($sContents, array $aAttr = array()) { return html::tag('span',   $sContents, $aAttr); } ///< Generate a <tt>\<span\></tt> tag.
	static function kbd   ($sContents, array $aAttr = array()) { return html::tag('kbd',    $sContents, $aAttr); } ///< Generate a <tt>\<kbd\></tt> tag.
	static function code  ($sContents, array $aAttr = array()) { return html::tag('code',   $sContents, $aAttr); } ///< Generate a <tt>\<code\></tt> tag.
	static function item  ($sContents, array $aAttr = array()) { return html::tag('li',     $sContents, $aAttr, true); } ///< Generate a <tt>\<li\></tt> tag.
	static function strong($sContents, array $aAttr = array()) { return html::tag('strong', $sContents, $aAttr); } ///< Generate a <tt>\<strong\></tt> tag.
	static function tr    ($sContents, array $aAttr = array()) { return html::tag('tr',     $sContents, $aAttr, true); } ///< Generate a <tt>\<tr\></tt> tag.
	static function td    ($sContents, array $aAttr = array()) { return html::tag('td',     $sContents, $aAttr); } ///< Generate a <tt>\<td\></tt> tag.
	static function th    ($sContents, array $aAttr = array()) { return html::tag('th',     $sContents, $aAttr); } ///< Generate a <tt>\<th\></tt> tag.
	static function b     ($sContents, array $aAttr = array()) { return html::tag('b',      $sContents, $aAttr); } ///< Generate a <tt>\<b\></tt> tag.
	static function i     ($sContents, array $aAttr = array()) { return html::tag('i',      $sContents, $aAttr); } ///< Generate an <tt>\<i\></tt> tag.
	
	
	/// Generate a <tt>\<p\></tt> tag having \c caption class.
	static function caption($sContents) { return self::par($sContents, attr::className('caption')); }
	
	
	/// Generate an <kbd>\<i\></kbd> tag for a text: apply html::text() to the contents.
	static function kbdText($sContents, array $aAttr = array())
	{
		return self::kbd(self::text($sContents), $aAttr);
	}
	
	/// Generate an <tt>\<acronym title="..."\>...\</acronym\></tt> tag.
	static function acronym($sAbbrev, $sTitle, array $aAttr = array())
	{
		$aAttr += attr::title($sTitle);
		return html::tag('acronym', $sAbbrev, $aAttr);
	}
	
	/// Generate an <tt>\<abbr title="..."\>...\</abbr\></tt> tag.
	static function abbr($sAbbrev, $sTitle, array $aAttr = array())
	{
		$aAttr += attr::title($sTitle);
		return html::tag('abbr', $sAbbrev, $aAttr);
	}
	
	/// Generate an <tt>\<a href="..."\>...\</a\></tt> tag.
	static function link($sTarget, $sCaption, array $aAttr = array())
	{
		$aAttr['href'] = $sTarget;
		return html::tag('a', $sCaption, $aAttr);
	}
	
	/// @brief Generate an <tt>\<a src="..." alt="..." width="..." height="..." /\></tt> anchor.
	///
	/// Give strictly negative size, for example <tt>$cx < 0</tt>,
	/// to disable automatic image size computing.
	static function img($sSource, $sAlt = '', $cx = null, $cy = null, array $aAttr = array())
	{
		if (!@parse_url($sSource, PHP_URL_SCHEME) && is_readable($sSource))
			Page::addCurrentCacheDependency($sSource, false);
		
		$aAttr += array('src' => $sSource, 'alt' => $sAlt);
		if ($cx >= 0 && $cy >= 0) {
			if ($cx !== null && $cy !== null) {
				$aAttr['width']  = $cx;
				$aAttr['height'] = $cy;
			}else if (strpos($sSource, '://') === false) {
				$aImageSize = getimagesize($sSource);
				$aAttr['width']  = $aImageSize[0];
				$aAttr['height'] = $aImageSize[1];
			}
		}
		return html::anchor('img', $aAttr);
	}
	
	/// @brief Generate an empty <tt>\<div\></tt> tag having \c clearempty class.
	/// The \c clearempty is supposed defined as following in CSS stylesheet:
	/// <tt>.clear { clear: both; font-size: 1px; line-height: 0px; height: 0px; }</tt>
	static function clear()
	{
		return html::div(html::nbsp(), attr::className('clearempty'));
	}
	
	/// Generate a <tt>\<hN\></tt> tag.
	static function title($iLevel, $sCaption, array $aAttr = array())
	{
		return html::tag('h' . $iLevel, $sCaption, $aAttr, true, true);
	}
	
	/// @}
	
	
	/// @name Table tags writing
	/// @{
	
	/// @brief Generate a <tt>\<table\></tt> tag.
	///
	/// @param $sContents an HTML string put in the <tt>\<table\></tt> tag,
	/// after the <tt>\<colgroup\></tt> tag, if any.
	/// @param $sColGroup if not empty, will be enclosed by a <tt>\<colgroup\></tt> tag
	/// and put inside the <tt>\<table\></tt> tag.
	/// @param $aAttr the attributes of the <tt>\<table\></tt> tag.
	static function tableBase($sContents, $sColGroup = '', array $aAttr = array())
	{
		if (!$sContents)
			return '';
		if ($sColGroup != '')
			$sColGroup = html::tag('colgroup', $sColGroup);
		return html::tag('table', $sColGroup . $sContents, $aAttr);
	}
	
	/// @brief Generate a <tt>\<table\></tt> tag.
	///
	/// @param $contents HTML string put as is in the <tt>\<table\></tt> tag,
	/// or rows array to be formatted by #trtdList().
	/// Will be put after the <tt>\<colgroup\></tt> tag and the header, if any.
	/// @param $sColGroup if not empty, will be enclosed by a <tt>\<colgroup\></tt> tag
	/// and put inside the <tt>\<table\></tt> tag.
	/// @param $aAttr the attributes of the <tt>\<table\></tt> tag.
	/// @param $sHead HTML string put after the <tt>\<colgroup\></tt> tag, if any,
	/// and before the contents. Intended to contain the result of a #trthList() call.
	static function table($contents, $sColGroup = '', array $aAttr = array(), $sHead = '')
	{
		$s = $sHead;
		$s .= (is_array($contents))
			? self::trtdGrid($contents)
			: $contents;
		return self::tableBase($s, $sColGroup, $aAttr);
	}
	
	/// @brief #table2() wrapper: use the keys of an array as fist column
	/// and its values as second column.
	static function table2(array $aRowAssoc, $sColGroup = '', array $aAttr = array(), $sHead = '')
	{
		return self::table(concatKeyVal($aRowAssoc), $sColGroup, $aAttr, $sHead);
	}
	
	/// @brief Generate a table row of <tt>\<td\></tt> tags from a cells list:
	/// <tt>\<tr\>\<td\>...\</td\>\<td\>...\</td\>...\</tr\></tt>.
	static function trtdList(array $aCell, array $aRowAttr = array())
	{
		return html::tr(html::mapConcat('td', $aCell), $aRowAttr);
	}
	
	/// @brief Generate a table row of <tt>\<th\></tt> tags from a cells list:
	/// <tt>\<tr\>\<th\>...\</th\>\<th\>...\</th\>...\</tr\></tt>.
	static function trthList(array $aCell, array $aRowAttr = array())
	{
		return html::tr(html::mapConcat('th', $aCell), $aRowAttr);
	}
	
	/// @brief Generate a sequence of rows of <tt>\<td\></tt> tags from a cells list:
	/// <tt>\<tr\>\<td\>...\</td\>\<td\>...\</td\>...\</tr\>\<tr\>...\</tr\>...</tt>.
	static function trtdGrid(array $aCell)
	{
		return html::mapConcat('trtdList', $aCell);
	}
	
	/// @brief Generate a <tt>\<col\></tt> anchor, with the given width (if not \c null) and attributes.
	static function col($sWidth = '', array $aAttr = array())
	{
		if (isSome($sWidth))
			$aAttr['width'] = $sWidth;
		return html::anchor('col', $aAttr);
	}
	
	/// @brief Generate a <tt>\<col\></tt> anchor having the given classname.
	static function colClass($sClassName)
	{
		return self::col(null, attr::className($sClassName));
	}
	
	/// @brief Generate a sequence of <tt>\<col\></tt> anchors having the specified widths.
	/// \c null widths generate an empty <tt>\<col\></tt> anchor.
	static function colsWidth(array $aWidth)
	{
		return self::mapConcat('col', $aWidth);
	}
	
	/// @}
	
	
	/// @name List tags writing
	/// @{
	
	/// Generate a sequence of <tt>\<li\></tt> tags from an array.
	static function items(array $aItem)
	{
		return self::mapConcat('item', $aItem);
	}
	
	/// @brief Generate a list tag (<tt>\<ul\></tt> or <tt>\<ol\></tt>)
	/// containing a sequence of items. The second argument can be an HTML string,
	/// or an items array to be formatted by #items().
	static function listGeneric($sTag, $contents, array $aAttr = array())
	{
		if (is_array($contents))
			$contents = html::items($contents);
		return ($contents)
			? html::tag($sTag, $contents, $aAttr)
			: '';
	}
	
	/// Generate a bullet list tag <tt>\<ul\></tt>.
	static function listU($contents, array $aAttr = array()) { return html::listGeneric('ul', $contents, $aAttr); }
	
	/// Generate a numbered list tag <tt>\<ul\></tt>.
	static function listO($contents, array $aAttr = array()) { return html::listGeneric('ol', $contents, $aAttr); }
	
	/// @}
	
	
	/// @name Header writing
	/// @{
	
	/// Generate a <tt>\<meta name="..." content="..." /\></tt> anchor.
	static function meta($sName, $sContent)
	{
		return (empty($sContent)) ? '' :
			html::anchor('meta', array('name' => $sName, 'content' => $sContent));
	}
	
	/// Generate a <tt>\<meta http-equiv="..." content="..." /\></tt> anchor.
	static function metaHTTP($sName, $sContent)
	{
		return (empty($sContent)) ? '' :
			html::anchor('meta', array('http-equiv' => $sName, 'content' => $sContent));
	}
	
	/// @brief Generate a <tt>\<link\></tt> anchor for a favicon.
	///
	/// The icon should exist in both ICO and PNG formats.
	/// @param $sPrefix the path of the icon, without file extension.
	static function icon($sPrefix)
	{
		return (isEmpty($sPrefix))
			? ''
			: html::anchor('link', array('rel' => 'shortcut icon', 'type' => 'image/x-icon', 'href' => $sPrefix . '.ico')) .
			  html::anchor('link', array('rel' => 'icon', 'type' => 'image/png', 'href' => $sPrefix . '.png'));
	}
	
	/// @brief Generate a link to an external CSS file (<tt>\<link/\></tt> anchor).
	static function cssLink($sCSS, array $aAttr = array())
	{
		return html::anchor('link', array('type' => 'text/css', 'rel' => 'stylesheet', 'href' => $sCSS) + $aAttr);
	}
	
	/// @brief Generate an internal CSS (<tt>\<style/\></tt> tag).
	static function cssInline($sCSS, array $aAttr = array())
	{
		return html::tag('style', regexp::replace('#[\r\n\t]+#', ' ', $sCSS), array('type' => 'text/css') + $aAttr);
	}
	
	/// @}
	
	
	/// @name Javascript writing
	/// @{
	
	/// @brief Generate a <tt>\<script\></tt> tag that includes a Javascript file.
	static function jscriptLink($sFileJS)
	{
		return html::tag('script', '', array('src' => $sFileJS, 'type' => 'text/javascript'));
	}
	
	/// @brief Generate a <tt>\<script\></tt> tag containing inline Javascript code.
	static function jscriptInline($sScript)
	{
		return html::tag('script', $sScript, array('type' => 'text/javascript'));
	}
	
	/// @brief Generate Javascript code to define a variable.
	/// The value is formatted using #JSON.
	static function jscriptSetVar($sName, $value)
	{
		return 'var ' . $sName . '=' . JSON::encode($value) . ';';
	}
	
	/// @brief Generate Javascript code to define a list of variables.
	/// The value is formatted using #JSON.
	static function jscriptSetVars(array $aVar)
	{
		return concat(mapKeyValue(array(__CLASS__, 'jscriptSetVar'), $aVar));
	}
	
	/// @brief Convert a string to a quoted Javascript string, by using JSON::encodeString().
	static function jscriptString($s)
	{
		return JSON::encodeString($s);
	}
	
	/// @}
	
	
	/// @brief Map a #html function to an array.
	static function map($sFunction, array $a)
	{
		return map(array(__CLASS__, $sFunction), $a);
	}
	
	/// @brief Map a #html function to an array, then concatenare the resulting strings.
	static function mapConcat($sFunction, array $a, $sSeparator = '')
	{
		return mapConcat(array(__CLASS__, $sFunction), $a, $sSeparator);
	}
}



//-----------------------------------------------------------------
/// Forms writing
//-----------------------------------------------------------------

class form
{
	const METHOD_GET = 1;
	const METHOD_POST = 2;
	const METHOD_MULTIPART = 3;
	
	static function formTo($sTarget, $iMethod, $sContents, array $aAttr = array(), array $aTargetParam = array())
	{
		$sHidden = '';
		foreach ($aTargetParam as $sParamName => $sParamValue)
			$sHidden .= self::hidden($sParamName, $sParamValue);
		if ($sHidden)
			$sContents .= html::div($sHidden);
		
		$aAttr['action'] = $sTarget;
		$aAttr['method'] = ($iMethod == self::METHOD_GET) ? 'get' : 'post';
		if ($iMethod == self::METHOD_MULTIPART)
			$aAttr['enctype'] = 'multipart/form-data';
	
		return html::tag('form', $sContents, $aAttr);
	}
	
	static function formSelf($iMethod, $sContents, array $aAttr = array(), array $aTargetParam = array())
	{
		return self::formTo(sSelfName, $iMethod, $sContents, $aAttr, $aTargetParam);
	}
	
	static function formJScript($sJScript, $sContents, array $aAttr = array())
	{
		return self::formTo('javascript:' . $sJScript, self::METHOD_GET, $sContents, $aAttr);
	}
	
	
	static function input($sType, $sName, $sValue = null, array $aAttr = array())
	{
		$aAttr += attr::nameid($sName);
		
		if (!is_null($sType))
			$aAttr['type'] = $sType;
		if (!is_null($sValue))
			$aAttr['value'] = (string)$sValue;
		
		return html::anchor('input', $aAttr);
	}
	
	static function textLike($sType, $sName, $sValue = '', $iSize = -1, $maxLength = -1, array $aAttr = array())
	{
		if ($iSize >= 0)
			$aAttr['size'] = $iSize;
		if ($maxLength >= 0)
			$aAttr['maxlength'] = $maxLength;
		return self::input($sType, $sName, $sValue, $aAttr);
	}
	
	static function text($sName, $sValue = '', $iSize = -1, $maxLength = -1, array $aAttr = array())
	{
		return self::textLike(null, $sName, $sValue, $iSize, $maxLength, $aAttr);
	}
	
	static function password($sName, $sValue = '', $iSize = -1, $maxLength = -1, array $aAttr = array())
	{
		return self::textLike('password', $sName, $sValue, $iSize, $maxLength, $aAttr);
	}
	
	static function hidden($sName, $sValue = '', array $aAttr = array())
	{
		return self::input('hidden', $sName, $sValue, $aAttr);
	}
	
	static function checkbox($sName, $bChecked = false, array $aAttr = array())
	{
		if ($bChecked)
			$aAttr['checked'] = null;
		return self::input('checkbox', $sName, null, $aAttr);
	}
	
	static function file($sName, $iSize = -1, array $aAttr = array())
	{
		if ($iSize >= 0)
			$aAttr['size'] = $iSize;
		return self::input('file', $sName, null, $aAttr);
	}
	
	static function label($sFor, $sContents, array $aAttr = array())
	{
		$aAttr['for'] = $sFor;
		return html::tag('label', $sContents, $aAttr);
	}
	
	static function checkboxLabel($sName, $sLabel, $bChecked = false, array $aAttrCheckbox = array(), array $aAttrLabel = array())
	{
		$sSearch = '/\[(.*)\]/';
		$aAccessKey = array();
		if (regexp::match($sSearch, $sLabel, $aAccessKey) == 1) {
			$sLabel = regexp::replace($sSearch, html::span('\1', attr::style('text-decoration: underline')), $sLabel);
			$sAccessKey = strtoupper($aAccessKey[1]);
			$aAttrLabel['accesskey'] = $sAccessKey;
		}
		
		return
			self::checkbox('ID' . $sName, $bChecked, $aAttrCheckbox) .
			self::label   (       $sName, $sLabel,   $aAttrLabel);
	}
	
	static function submit($sName, $sLabel, array $aAttr = array())
	{
		return self::input('submit', $sName, $sLabel, $aAttr);
	}
	
	static function image($sImageUrl, $sName, array $aAttr = array())
	{
		return self::input('image', $sName, null, $aAttr + array('src' => $sImageUrl));
	}
	
	static function button($sName, $sLabel, $sOnClick = '', array $aAttr = array())
	{
		if ($sOnClick)
			$aAttr['onclick'] = $sOnClick;
		return self::input('button', $sName, $sLabel, $aAttr);
	}
	
	static function textArea($sName, $nbCol, $nbRow, $sContents = '', array $aAttr = array())
	{
		$aAttr += attr::nameid($sName) + array('cols' => $nbCol, 'rows' => $nbRow);
		return html::tag('textarea', $sContents, $aAttr);
	}
	
	static function fieldset($sLegend, $sContents, array $aAttr = array(), $aAttrLegend = array())
	{
		return html::tag('fieldset',
			html::tag('legend', $sLegend, $aAttrLegend) . $sContents,
			$aAttr);
	}
	
	static function select($sName, $aChoice, $defaultKey = null, array $aAttr = array())
	{
		$sOptions = '';
		foreach ($aChoice as $key => $sValue) {
			$aOptionAttr = array('value' => $key);
			if ($key == $defaultKey)
				$aOptionAttr['selected'] = 1;
			$sOptions .= html::tag('option', $sValue, $aOptionAttr);
		}
		return html::tag('select', $sOptions, $aAttr + attr::nameid($sName));
	}
}



//-----------------------------------------------------------------
/// Frameset page writing
//-----------------------------------------------------------------

class htmlframeset
{
	const sDocType = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN" "http://www.w3.org/TR/html4/frameset.dtd">';
	
	
	static function tag($sName, $sContents, array $aAttr = array())
	{
		return '<' . $sName . attr::toString($aAttr) . '>' . $sContents . '</' . $sName . '>';
	}
	
	static function anchor($sName, array $aAttr = array())
	{
		return '<' . $sName . attr::toString($aAttr) . '>';
	}
	
	static function convertAnchors($s)
	{
		return str_replace('/>', '>', $s);
	}
	
	
	static function frameset($sContents, $sRows = '', $sCols = '', array $aAttr = array())
	{
		if ($sRows)
			$aAttr['rows'] = $sRows;
		if ($sCols)
			$aAttr['cols'] = $sCols;
		
		return self::tag('frameset', $sContents, $aAttr);
	}
	
	static function frame($sSrc, $sName = '', array $aAttr = array())
	{
		$aAttr['src'] = $sSrc;
		if ($sName)
			$aAttr['name'] = $sName;
		
		return self::anchor('frame', $aAttr);
	}
}

?>