<?php defined('KERNEL_INCLUDED') or die('kernel should be included first');
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder

//-----------------------------------------------------------------
/// @file
/// @brief #sql: SQL wrapper
//-----------------------------------------------------------------

/// @brief If \c true, maintain the history of all executed queries
/// in the array sql::$aQuery.
defineDefault('bDebugSQL', false);


/// @brief Flags for sql::dumpTables() function.
defineFlags('sqlDump', 'Data;Structure;DropIfExists');

/// @brief Dump both data and structure.
define('sqlDumpAll', sqlDumpData | sqlDumpStructure);

/// @brief Default dumping settings: dump data and structure, drop tables before if they exist.
define('sqlDumpDefault', sqlDumpAll | sqlDumpDropIfExists);


//-----------------------------------------------------------------
/// @brief SQL wrapper
///
/// Table and field name quoting is not performed by the functions.
//-----------------------------------------------------------------

class sql
{
	/// The number of queries executed so far.
	public static $nbQuery = 0;
	
	/// @brief The list of all queries executed so far.
	/// This array is populated only if #bDebugSQL is \c true.
	public static $aQuery = array();
	
	
	/// @brief Set the SQL connection information,
	/// by defining the constants \c sql_sHost, \c sql_sUser, \c sql_sPassword, \c sql_sBase.
	/// Must be called before the first call to connect().
	public static function setup($sHost, $sUser, $sPassword, $sBase)
	{
		defineArray(array(
 			'sql_sHost'     => $sHost,
			'sql_sUser'     => $sUser,
			'sql_sPassword' => $sPassword,
			'sql_sBase'     => $sBase,
		));
	}
	
	
	/// @brief Create the SQL connection, using the parameters given
	/// by the last call of setup(). This method can only been called once.
	public static function connect()
	{
		static $bConnected = false;
		if ($bConnected)
			return;
		
		$sHost     = sql_sHost;
		$sUser     = sql_sUser;
		$sPassword = sql_sPassword;
		$sBase     = sql_sBase;
		
		@mysql_connect($sHost, $sUser, $sPassword)
			or debug::fatalError('Unable to connect to database server: ' . mysql_error());
		
		@mysql_select_db($sBase)
			or debug::fatalError('Unable to connect to database: ' . mysql_error());
		
		$bConnected = true;
	}
	
	
	/// @brief Execute a query, return its result (a resource object).
	/// If an error occurs, raise a fatal error and exit.
	///
	/// @param $sQuery the SQL query to execute.
	/// @return the request result as a resource object, never \c null.
	public static function query($sQuery)
	{
		if (self::$nbQuery++ == 0)
			self::connect();
		
		if (bDebugSQL)
			self::$aQuery[] = $sQuery;
		
		$res = mysql_query($sQuery);
		$res or debug::fatalError('Unable to execute query: ' . mysql_error(), $sQuery);
		return $res;
	}
	
	/// Return the number of rows of the last result.
	public static function getCount($res)
	{
		return (int)@mysql_num_rows($res);
	}
	
	/// Return the number of rows affected by the last request.
	public static function getAffectedCount()
	{
		return (int)@mysql_affected_rows();
	}
	
	/// Free all information attached to the given request result.
	public static function free($res)
	{
		@mysql_free_result($res);
	}
	
	/// <tt>$sAlias => $sName</tt> array to <tt>$sName . ' AS ' . $sAlias</tt> array.
	/// Escape a SQL string by inserting backslashes.
	public static function text($s)
	{
		return '\'' . mysql_real_escape_string($s) . '\'';
	}
	
	/// @brief Convert an associative array <tt>('alias' => 'name')</tt>
	/// or <tt>(integer => 'name')</tt> to a string array.
	/// Elements having a string key <tt>('alias' => 'name')</tt>
	/// are converted to <tt>'alias TO name'</tt>,
	/// all other elements are kept as is.
	public static function aliasListFormat(array $a)
	{
		$aName = array();
		foreach ($a as $sAlias => $sName)
			$aName[] = (is_string($sAlias))
				? $sName . ' AS ' . $sAlias
				: $sName;
		return $aName;
	}
	
	/// Same as aliasListFormat(), but the result array
	/// is converted to a \c ',' separated string.
	public static function aliasListToString(array $a)
	{
		return concat(self::aliasListFormat($a), ', ');
	}
	
	
	/// Convert a tables array or string to a string.
	///
	/// @param $tables if it is an array, apply it to aliasListToString(),
	/// otherwise returned it as is.
	public static function tablesToString($tables)
	{
		return (is_array($tables))
			? self::aliasListToString($tables)
			: $tables;
	}
	
	/// Convert a fields array to a string, by using aliasListToString().
	public static function fieldsNameToString(array $aField)
	{
		return self::aliasListToString($aField);
	}
	
	
	/// @brief Execute a \c SELECT query and return its result as a resource object.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $aField the fields to select, to be formatted by fieldsNameToString().
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE and/or a <tt>ORDER BY</tt> clause.
	/// @return the request result as a resource object, never \c null.
	public static function select($tables, array $aField = array('*'), $sMore = '')
	{
		return self::query(
			'SELECT ' . self::fieldsNameToString($aField) .
			' FROM ' . self::tablesToString($tables) .
			' ' . $sMore);
	}
	
	
	/// @brief Execute a one-result \c SELECT query, in the same way than select().
	/// Return the request resource, or \c null if not exactly one result is returned.
	/// @see select()
	public static function selectOne($tables, array $aField = array('*'), $sMore = '')
	{
		$res = self::select($tables, $aField, $sMore);
		if (self::getCount($res) == 1)
			return $res;
	
		self::free($res);
		return null;
	}
	
	/// @brief Execute a one-result \c SELECT query with selectOne().
	/// Return its unique result as an object with fetchFree(), or \c null if error.
	/// @see selectOne()
	public static function selectOneFetch($tables, array $aField = array('*'), $sMore = '', $sClass = 'stdClass')
	{
		$res = self::selectOne($tables, $aField, $sMore);
		return ($res) ? self::fetchFree($res, $sClass) : null;
	}
	
	/// @brief Execute a one-result \c SELECT query with selectOne().
	/// Return its result for a single field, or \c null if error.
	/// @see selectOneFetch()
	public static function selectOneField($tables, $sFieldOrExpr, $sMore = '')
	{
		$obj = self::selectOneFetch($tables, array('sSqlResult' => $sFieldOrExpr), $sMore);
		return ($obj) ? $obj->sSqlResult : null;
	}
	
	/// @brief Execute a multi-result \c SELECT query.
	/// Return its results as an object array, or an empty array if error.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $aField the fields to select, to be formatted by fieldsNameToString().
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE and/or a <tt>ORDER BY</tt> clause.
	/// @param $sClass the name of the class of the elements of the returned array,
	/// instantiated for each row.
	/// @return an array containing objects of type \p $sClass.
	/// @see select()
	public static function selectFetch($tables, array $aField = array('*'), $sMore = '', $sClass = 'stdClass')
	{
		$res = self::select($tables, $aField, $sMore);
		
		$aResult = array();
		if ($res) {
			// Major optimization: because we may fetch very large array,
			// using fetch() would take much time (type casting, names loading, etc.).
			// Here we unfold the loop by generating PHP code to load all results into $aResult array.
			// The resulting code is almost as fast as a loop using mysql_fetch_object(),
			// but it checks types and create objects of $sClass class.
			
			$nbField = mysql_num_fields($res);
			
			$sCode =
				'while (($aValue = mysql_fetch_row($res)) !== false) {' . CRLF .
					TAB . '$obj = new ' . $sClass . ';' . CRLF;
			for ($iField = 0; $iField < $nbField; $iField++) {
				$sName = mysql_field_name($res, $iField);
				$sType = deduceType($sName);
				
				$sCode .= TAB . '$value = $aValue[' . $iField . '];' . CRLF;
				if ($sType)
					$sCode .= TAB . 'settype($value, \'' . $sType . '\');' . CRLF;
				$sCode .= TAB . '$obj->' . $sName . ' = $value;' . CRLF;
			}
			$sCode .= TAB . '$aResult[] = $obj;' . CRLF;
			$sCode .= '}' . CRLF;
			
			// Debugging:
			// echo html::pre(html::text($sCode));
			
			eval($sCode);
			
			self::free($res);
		}
	
		return $aResult;
	}
	
	/// @brief Execute a multi-result \c SELECT query.
	/// Return its results as an object associative array
	/// whose key is \p $sFieldBy field, or an empty array if error.
	///
	/// Unlike selectFetchListBy(), this function does not support the case
	/// where several rows have the same key. For a given key, only the last row
	/// having this key is kept. Rows may be lost: the returned array
	/// may have less elements than raw the request result.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $sFieldBy the name of the field giving the keys of the returned array,
	/// must be an element of \p $aField.
	/// @param $aField the fields to select, to be formatted by fieldsNameToString().
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE and/or a <tt>ORDER BY</tt> clause.
	/// @param $sClass the name of the class of the elements of the returned array,
	/// instantiated for each row.
	/// @return an array containing objects of type \p $sClass.
	///
	/// @see selectFetchListBy(), selectFetch()
	public static function selectFetchBy($tables, $sFieldBy = 'id', array $aField = array('*'), $sMore = '', $sClass = 'stdClass')
	{
		$aResult = array();
		foreach (self::selectFetch($tables, $aField, $sMore, $sClass) as $result)
			$aResult[$result->$sFieldBy] = $result;
		return $aResult;
	}
	
	/// @brief Execute a multi-result \c SELECT query.
	/// Return its results as an associative array containing for each key,
	/// given by the \p $sFieldBy field, the list of objects matching this key.
	///
	/// Unlike selectFetchBy(), this function keeps all rows does not support the case
	/// where several rows have the same key. For a given key, all rows having this key
	/// are returned. No row is lost.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $sFieldBy the name of the field giving the keys of the returned array.
	/// must be an element of \p $aField.
	/// @param $aField the fields to select, to be formatted by fieldsNameToString().
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE and/or a <tt>ORDER BY</tt> clause.
	/// @param $sClass the name of the class of the elements of the returned array,
	/// instantiated for each row.
	/// @return an array containing objects of type \p $sClass.
	///
	/// @see selectFetchBy(), selectFetch()
	public static function selectFetchListBy($tables, $sFieldBy = 'id', array $aField = array('*'), $sMore = '', $sClass = 'stdClass')
	{
		$aResult = array();
		foreach (self::selectFetch($tables, $aField, $sMore, $sClass) as $result)
			$aResult[$result->$sFieldBy][] = $result;
		return $aResult;
	}
	
	
	/// @brief Execute a multi-result \c SELECT query.
	/// Return its results as an array containing the value of a field.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $sField the name of the field containing the values to return.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE and/or a <tt>ORDER BY</tt> clause.
	/// @return an array containing the values of the \p $sField field.
	/// @see selectFetch()
	public static function selectField($tables, $sField, $sMore = '')
	{
		$aResult = array();
		foreach (self::selectFetch($tables, array($sField), $sMore) as $result)
			$aResult[] = $result->$sField;
		return $aResult;
	}
	
	/// @brief Execute a multi-result \c SELECT query.
	/// Return its results as an array associating a field to another.
	///
	/// If several rows have the same key, only the value of the last row
	/// having the key is kept.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $sFieldKey the name of the field giving the keys of the array.
	/// @param $sFieldValue the name of the field giving the values of the array.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE and/or a <tt>ORDER BY</tt> clause.
	/// @return an array associating the values of the \p $sFieldValue field
	/// to the values of the \p $sFieldKey field.
	/// @see selectFetch()
	public static function selectFieldBy($tables, $sFieldKey, $sFieldValue, $sMore = '')
	{
		$aResult = array();
		foreach (self::selectFetch($tables, array($sFieldKey, $sFieldValue), $sMore) as $result)
			$aResult[$result->$sFieldKey] = $result->$sFieldValue;
		return $aResult;
	}
	
	
	/// @brief Return the number of results of a \c SELECT query.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	public static function count($tables, $sMore = '')
	{
		return (int)self::selectOneField($tables, 'COUNT(*)', $sMore);
	}
	
	
	/// @brief Execute an \c UPDATE query.
	///
	/// @param $tables the array or string specifying the tables of the \c FROM clause,
	/// to be formatted by tablesToString().
	/// @param $aData the values to set with a \c SET clause,
	/// to be formatted by formatFields(). Keys are field name, values are field value.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @return the request result as a resource object, never \c null.
	public static function update($tables, array $aData, $sMore = '')
	{
		return self::query('UPDATE ' . self::tablesToString($tables) .
			' SET ' . self::formatFields($aData) . " $sMore");
	}
	
	
	/// @brief Execute an <tt>INSERT INTO</tt> query to insert a single row,
	/// taking the row data from an array.
	///
	/// @param $sTable the name of the table where to insert the row.
	/// @param $aData an associative array containing the values to use in the \c SET clause,
	/// to be formatted by formatFields(). Keys are field name, values are field value.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @return the generated AUTO_INCREMENT of the inserted row, if any.
	/// @see insertOrReplace()
	public static function insertInto($sTable, array $aData, $sMore = '')
	{
		return self::insertOrReplace('INSERT INTO', $sTable, $aData, $sMore);
	}
	
	/// @brief Execute an <tt>INSERT INTO</tt> query to insert a single row,
	/// taking the row data from an object.
	///
	/// @param $sTable the name of the table where to insert the row.
	/// @param $obj the object containing the values to use in the \c SET clause.
	/// @param $aField the list of the variable members of \p $obj
	/// to put in the \c SET clause, <tt>array('*')</tt> to use all fields.
	/// Used by getObjectFields() function.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @return the generated AUTO_INCREMENT of the inserted row, if any.
	/// @see insertInto()
	public static function insertIntoObj($sTable, $obj, array $aField = array('*'), $sMore = '')
	{
		return self::insertInto($sTable, getObjectFields($obj, $aField), $sMore);
	}
	
	/// @brief Execute an <tt>INSERT INTO</tt> query to insert several rows,
	/// taking the rows data from an array.
	///
	/// @param $sTable the name of the table where to insert the row.
	/// @param $aField the name of the fields,
	/// must have as many elements as every row of \p $aDataList.
	/// @param $aDataList an array of associative arrays containing all rows to insert.
	/// All rows must be contained in an associative array.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @see insertOrReplaceMultiple()
	public static function insertIntoMultiple($sTable, array $aField, array $aDataList, $sMore = '')
	{
		return self::insertOrReplaceMultiple('INSERT INTO', $sTable, $aField, $aDataList, $sMore);
	}
	
	
	/// @brief Execute a \c REPLACE query to replace a single row,
	/// taking the row data from an array.
	///
	/// @param $sTable the name of the table where to replace the row.
	/// @param $aData an associative array containing the values to use in the \c SET clause,
	/// to be formatted by formatFields(). Keys are field name, values are field value.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @return the generated AUTO_INCREMENT of the inserted row, if any.
	/// @see insertOrReplace()
	public static function replace($sTable, array $aData, $sMore = '')
	{
		return self::insertOrReplace('REPLACE', $sTable, $aData, $sMore);
	}
	
	/// @brief Execute a \c REPLACE query to replace a single row,
	/// taking the row data from an object.
	///
	/// @param $sTable the name of the table where to replace the row.
	/// @param $obj the object containing the values to use in the \c SET clause.
	/// @param $aField the list of the variable members of \p $obj
	/// to put in the \c SET clause, <tt>array('*')</tt> to use all fields.
	/// Used by getObjectFields() function.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @return the generated AUTO_INCREMENT of the inserted row, if any.
	/// @see replace()
	public static function replaceObj($sTable, $obj, array $aField = array('*'), $sMore = '')
	{
		return self::replace($sTable, getObjectFields($obj, $aField), $sMore);
	}
	
	/// @brief Execute a \c REPLACE query to replace several rows,
	/// taking the rows data from an array.
	///
	/// @param $sTable the name of the table where to replace the row.
	/// @param $aField the name of the fields,
	/// must have as many elements as every row of \p $aDataList.
	/// @param $aDataList an array of associative arrays containing all rows to insert.
	/// All rows must be contained in an associative array.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @see insertOrReplaceMultiple()
	public static function replaceMultiple($sTable, array $aField, array $aDataList, $sMore = '')
	{
		return self::insertOrReplaceMultiple('REPLACE', $sTable, $aField, $aDataList, $sMore);
	}
	
	
	/// @brief Execute an <tt>INSERT INTO</tt> or a \c REPLACE query to insert/replace a single row,
	/// taking the row data from an array.
	///
	/// @param $sInstruction the SQL instruction to execute: <tt>'INSERT INTO'</tt> or \c 'REPLACE'.
	/// @param $sTable the name of the table where to insert the row.
	/// @param $aData an associative array containing the values to use in the \c SET clause,
	/// to be formatted by formatFields(). Keys are field name, values are field value.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @return the generated AUTO_INCREMENT of the inserted row, if any.
	/// @see insert(), replace()
	public static function insertOrReplace($sInstruction, $sTable, array $aData, $sMore = '')
	{
		self::query($sInstruction . ' ' . $sTable .
			' (' . concat(array_keys($aData), ', ') . ')' .
			' VALUES (' . mapConcatKeyValue(array(__CLASS__, 'formatValue'), $aData, ', ') . ') ' . $sMore);
		return mysql_insert_id();
	}
	
	/// @brief Execute an <tt>INSERT INTO</tt> query to insert/replace several rows,
	/// taking the rows data from an array.
	///
	/// @param $sInstruction the SQL instruction to execute: <tt>'INSERT INTO'</tt> or \c 'REPLACE'.
	/// @param $sTable the name of the table where to insert/replace the row.
	/// @param $aField the name of the fields,
	/// must have as many elements as every row of \p $aDataList.
	/// @param $aDataList an array of associative arrays containing all rows to insert.
	/// All rows must be contained in an associative array.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	/// @see insertMultiple(), replaceMultiple()
	public static function insertOrReplaceMultiple($sInstruction, $sTable, array $aField, array $aDataList, $sMore = '')
	{
		if (!$aDataList)
			return;
		
		// Format the data in $aDataList
		$sCode = 'foreach ($aDataList as &$aData) {' . CRLF;
		foreach ($aField as $i => $sField) {
			$sElem = '$aData[' . $i . ']';
			switch (deduceType($sField)) {
				
				case 'integer':
					$sExpr = 'intval(' . $sElem . ')';
					break;
				
				case 'boolean':
					$sExpr = 'toInt(' . $sElem . ')';
					break;
				
				case 'string':
					$sExpr = 'self::text(' . $sElem . ')';
					break;
				
				default:
					continue;
			}
			
			$sCode .= TAB . $sElem . ' = ' . $sExpr . ';' . CRLF;
		}
		$sCode .= TAB . '$aData = \'(\' . implode($aData, \',\') . \')\';' . CRLF;
		$sCode .= '}' . CRLF;
		
		// Debugging:
		// echo html::pre(html::text($sCode));
		
		eval($sCode);
		
		self::query($sInstruction . ' ' . $sTable .
			' (' . concat($aField, ', ') . ')' .
			' VALUES ' . concat($aDataList, ', ') . ' ' . $sMore);
	}
	
	
	/// @brief Execute a \c DELETE query on a table.
	///
	/// @param $sTable the name of the table where to replace the row.
	/// @param $sMore string to be added at the end of the request.
	/// Typically contains a \c WHERE clause.
	public static function delete($sTable, $sMore = '')
	{
		return self::query('DELETE FROM ' . $sTable . ' ' . $sMore);
	}
	
	
	/// Convert a PHP value to a SQL-compatible string,
	/// by using a field name to get information about the value type.
	///
	/// The performed conversion is chosen only by analyzing the field name prefix.
	/// The effective PHP type of the value is ignored.
	/// - integers are converted to a numeric value
	/// - booleans are converted to a 0/1 value
	/// - strings are properly quoted
	/// - unrecognized types are passed as is
	///
	/// @param $sField the field name. Must have a supported prefix,
	/// eventually after a dot <tt>'.'</tt>.
	/// @param $value the value to convert to a SQL-compatible string.
	/// @see deduceType()
	public static function formatValue($sField, $value)
	{
		$i = strrpos($sField, '.');
		switch (deduceType(($i === false) ? $sField : substr($sField, $i + 1))) {
			
			case 'integer':
				return intval($value);
			
			case 'boolean':
				return toInt($value);
			
			case 'string':
				return self::text($value);
			
			default:
				return $value;
		}
	}
	
	
	/// Format a <tt>field=value</tt> SQL clause.
	///
	/// @param $sField the field name. If it begins with a \c '!',
	/// \p $value remains untouched: no type casting, no quotes.
	/// @param $value the value to put in the clause.
	/// If \p $sField does not begin with a \c '!',
	/// it will be casted and eventually quoted by formatValue()
	/// according to the prefix of \p $sField.
	/// @see formatValue()
	public static function formatField($sField, $value)
	{
		if ($value === null)
			return $sField;
		
		if ($sField[0] == '!')
			return substr($sField, 1) . '=' . $value;
		
		return $sField . '=' . self::formatValue($sField, $value);
	}
	
	/// @brief Format an associative array <tt>(field => value)</tt>
	/// to an SQL clause: <tt>field1=value1, field2=value2, ...</tt>.
	///
	/// @param $a the array containing the <tt>(field => value)</tt> associations.
	/// @return a \c ',' separated string of SQL clauses computed by formatField().
	public static function formatFields($a)
	{
		return mapConcatKeyValue(array(__CLASS__, 'formatField'), $a, ', ');
	}
	
	
	/// @brief Format a <tt>expr IN (a1, ..., aN)</tt> clause,
	/// where every <tt>ai</tt> is an integer.
	///
	/// @param $sExpr the expression whose value must be
	/// in the list given by \p $aValue.
	/// @param $aValue a list of integer values, may be empty.
	public static function formatInIntList($sExpr, array $aValue)
	{
		return ($aValue)
			? $sExpr . ' IN (' . mapConcat('intval', $aValue, ',') . ')'
			: '0';
	}
	
	
	/// Format a \c WHERE clause: <tt>WHERE field1=value1 AND field2=value2, ...</tt>.
	///
	/// @param $a an associative array <tt>(field => value)</tt>.
	public static function where(array $a)
	{
		if (!$a)
			return '';
		return 'WHERE ' . mapConcatKeyValue(array(__CLASS__, 'formatField'), $a, ' AND ');
	}
	
	/// Same as where() but for one condition only
	public static function whereOne($sField, $value)
	{
		return 'WHERE ' . self::formatField($sField, $value);
	}
	
	/// Same as where() but for one condition concerning 'ID' field only
	public static function whereId($id)
	{
		return 'WHERE id=' . (int)$id;
	}
	
	
	/// Fetch the current row of a result to an object of type \p $sClass.
	public static function fetch($res, $sClass = 'stdClass')
	{
		$obj = new $sClass;
		$objTemp = mysql_fetch_object($res);
		foreach ($objTemp as $sKey => &$value) {
			forceType($value, $sKey);
			$obj->$sKey = $value;
		}
		return $obj;
	}
	
	/// Same as fetch(), but free the resource with free() after fetching.
	public static function fetchFree($res, $sClass = 'stdClass')
	{
		$obj = self::fetch($res, $sClass);
		self::free($res);
		return $obj;
	}
	
	
	/// @brief Lock tables for exclusive access. Other threads wanting to
	/// reading or write on the given tables will be paused.
	///
	/// Each call of lockTables() must specify all tables to lock.
	/// When lockTables() is called several times, all tables are first
	/// unlocked before possibly locked again by lockTables().
	///
	/// @param $aTableExclusiveAccess tables to lock for both writing AND reading.
	/// Only the caller thread can write into or read in the tables.
	/// @param $aTableOnlyRead tables to lock for reading.
	/// Any thread can read the table, none can write.
	public static function lockTables(array $aTableExclusiveAccess, array $aTableOnlyRead = array())
	{
		$aTable = array_merge(
			self::lockAliasListToString($aTableExclusiveAccess, 'WRITE'),
			self::lockAliasListToString($aTableOnlyRead,        'READ'));
		
		self::query('LOCK TABLES ' . concat($aTable, ','));
	}
	
	/// Unlock the tables previously locked by lockTables().
	public static function unlockTables()
	{
		self::query('UNLOCK TABLES');
	}
	
	/// @brief Convert a list of possibly aliased table names
	/// to a list of strings with aliasListFormat(),
	/// and insert a suffix to all elements of it.
	/// Return the resulting array.
	private static function lockAliasListToString(array $a, $sSuffix)
	{
		$sSuffix = ' ' . $sSuffix;
		$aResult = array();
		foreach (self::aliasListFormat($a) as $sTable)
			$aResult[] = $sTable . $sSuffix;
		return $aResult;
	}
	
	
	/// @brief Return an array containing the name of all tables of the active database.
	public static function getTables()
	{
		$aTable = array();
		
		$res = self::query('SHOW TABLES');
		while (($aValue = mysql_fetch_row($res)) !== false)
			$aTable[] = $aValue[0];
		self::free($res);
		
		return $aTable;
	}
	
	/// @brief Convert a tables list (array or <tt>';'</tt> separated list) to an array of names.
	/// If no tables list is given, return all tables of the database.
	public static function tablesToArray($tables)
	{
		return ($tables === null)
			? self::getTables()
			: getListSafe($tables);
	}
	
	/// @brief Optimize the given tables. The tables list is given by tablesToArray().
	public static function optimizeTables($tables = null)
	{
		sql::free(self::query(
			'OPTIMIZE TABLES ' . concat(', ', self::tablesToArray($tables))));
	}
	
	
	/// @brief Dump SQL code of a table, directly on stdout.
	public static function dumpTable($sTable, $iFlags = sqlDumpDefault)
	{
		if ($iFlags & sqlDumpStructure) {
			if ($iFlags & sqlDumpDropIfExists)
				self::dumpLine('DROP TABLE IF EXISTS ' . $sTable . ';');
			
			$res = self::query('SHOW CREATE TABLE ' . $sTable . ';');
			$aValue = mysql_fetch_assoc($res);
			self::dumpLine($aValue['Create Table'] . ';');
			self::free($res);
		}
		
		if ($iFlags & sqlDumpData) {
			$res = sql::select($sTable);
			
			self::dumpLine('INSERT INTO ' . $sTable . ' VALUES');
			
			// Major optimization: because we may fetch very large array,
			// using fetch() would take much time (type casting, names loading, etc.).
			// Here we unfold the loop by generating PHP code to load all results into $aResult array.
			// The resulting code is almost as fast as a loop using mysql_fetch_object(),
			// but it checks types and create objects of $sClass class.
			
			$nbField = mysql_num_fields($res);
			$iLastRow = self::getCount($res) - 1;
			
			$sCode =
				'for ($iRow = 0; $iRow <= $iLastRow; $iRow++) {' . CRLF .
				TAB . '$aValue = mysql_fetch_row($res);' . CRLF .
				TAB . '$s = \'(\';' . CRLF;
			for ($iField = 0; $iField < $nbField; $iField++) {
				$field = mysql_fetch_field($res, $iField);
				
				$sField = ($field->type == 'int')
					? '$value'
					: '\'\\\'\' . mysql_real_escape_string($value) . \'\\\'\'';
				
				if ($iField > 0)
					$sCode .= TAB . '$s .= \',\';' . CRLF;
				
				$sCode .=
					TAB . '$value = $aValue[' . $iField . '];' . CRLF .
					TAB . '$s .= (is_null($value))' . CRLF .
					TAB . TAB . '? \'null\'' . CRLF .
					TAB . TAB . ': ' . $sField . ';' . CRLF;
			}
			$sCode .=
				TAB . 'echo $s . \')\' . (($iRow < $iLastRow) ? \',\' : \';\') . LF;' . CRLF .
				'}' . CRLF;
			
			// Debugging:
			// echo $sCode;
			
			eval($sCode);
			
			self::free($res);
		}
	}
	
	/// @brief Dump the tables whose names are given as an array or a <tt>';'</tt> separated list.
	/// If no tables list is given, dump all tables of the database.
	public static function dumpTables($tables = null, $iFlags = sqlDumpDefault)
	{
		foreach (self::tablesToArray($tables) as $sTable)
			self::dumpTable($sTable, $iFlags);
	}
	
	
	/// @brief Output a single line of a database dump.
	private static function dumpLine($sLine)
	{
		echo $sLine . LF;
	}
}

?>