<?php defined('KERNEL_INCLUDED') or die('kernel should be included first');
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

//-----------------------------------------------------------------
/// @file
/// @brief #PageCategories: class for pages displaying a list of categories,
/// that the user can open or close individually
//-----------------------------------------------------------------


//-----------------------------------------------------------------
/// @brief Base class for pages displaying a list of categories,
/// that the user can open or close individually
//-----------------------------------------------------------------

abstract class PageCategories extends PageDenif
{
	/// ID of the category to be initially opened.
	/// @see getPropertyAssoc
	public $id;
	
	
	protected function getParameters() { return concatLists(parent::getParameters(), 'id'); }
	
	
	protected function scripts()
	{
		return
			parent::scripts() .
			html::jscriptLink('global.js') .
			html::jscriptLink('categories.js');
		return $s;
	}
	
	
	protected function initialize()
	{
		parent::initialize();
		$this->loadParameters();
	}
	
	
	protected static function getTable(array $aCell, array $aColWidth, $sClassNameMore = 'cat-contents', array $aAttr = array())
	{
		$sClassName = 'sublist';
		if ($sClassNameMore)
			$sClassName .= ' ' . $sClassNameMore;
		
		return html::table($aCell, html::colsWidth($aColWidth), attr::className($sClassName) + $aAttr);
	}
	
	
	/// @brief Print all resources in a categories list, using a property as category key.
	///
	/// <tt>$this->id</tt> indicates the categories to initially open:
	/// - if <tt>\< 0</tt>: no resource is initially opened;
	/// - if <tt>= 0</tt>: all resources are initially opened;
	/// - if <tt>\> 0</tt>: specifies the ID of the resource to initially open.
	///
	/// @param $iPropertyKey the property giving the name of the categories.
	/// @param $iPropertyValue the property printed near each resource.
	/// @param[out] $aKey the \p $iPropertyKey properties, get by the function.
	/// @param $aInitiallyCheckedById if not \c null, each resource has a checkbox.
	/// The keys of the array give the initially checked resources.
	protected function getPropertyAssoc($iPropertyKey, $iPropertyValue, array &$aKey, $aInitiallyCheckedById = null)
	{
		$type = ObjectTypeInfo::get($iPropertyKey);
		$bCheckBoxes = ($aInitiallyCheckedById !== null);
		
		$aKey           = getProperties($iPropertyKey);
		$aKeyResource   = property2node($iPropertyKey, nodeResource);
		$aResource      = getNodes(nodeResource);
		$aResourceKey   = node2property(nodeResource, $iPropertyKey);
		$aResourceValue = node2property(nodeResource, $iPropertyValue);
		$aValue         = getProperties($iPropertyValue);
		
		$bAdmin = $this->isAdmin();
		
		// Compute flags, remove invisible resources
		$nodeDefault = node::getDefault();
		foreach ($aResource as $resource) {
			$resource->computeInheritedFlags($nodeDefault);
			if (!$bAdmin && $resource->aFlagInherit[nodeFlagVisible] != visibleYes)
				unset($aResource[$resource->id]);
		}
		
		$aColWidth = array(400, null);
		if ($bCheckBoxes)
			array_unshift($aColWidth, null);
		
		// Create a category for ressources having no key
		$aResourceNoKey = array();
		foreach ($aResource as $idResource => $resource)
			if (!isset($aResourceKey[$idResource]))
				$aResourceNoKey[] = $idResource;
		if ($aResourceNoKey) {
			$keyDefault = new stdClass;
			$keyDefault->id = 0;
			$aKey[] = $keyDefault;
		}
		
		$sKeyList = '';
		foreach ($aKey as $key) {
			$idKey = $key->id;
			
			$aRow = array();
			$a = array();
			if ($idKey) {
				if (!empty($aKeyResource[$idKey]))
					$a = $aKeyResource[$idKey];
			}else
				$a = $aResourceNoKey;
			
			foreach ($a as $idResource) {
				if (isset($aResource[$idResource])) {
					$aLinkValue = array();
					if (!empty($aResourceValue[$idResource]))
						foreach ($aResourceValue[$idResource] as $idValue)
							$aLinkValue[] = $aValue[$idValue]->link($this);
					
					// Resource icons & link and values links
					$resource = $aResource[$idResource];
					$aLink = $this->getObjectLinks($resource, '', objectLinksDelete);
					append($aLink, $this->getNodeFlagsIcon($resource));
					$aLink[] = $resource->link($this);
					$aCell = array(concat($aLink), concat($aLinkValue, ' | '));
					
					// Resource checkbox
					if ($bCheckBoxes)
						array_unshift($aCell, form::checkbox('_' . $idResource,
							array_key_exists($idResource, $aInitiallyCheckedById)));
					
					$aRow[] = $aCell;
				}
			}
			
			// Category title
			$aAttrTitle = attr::className('cat-title');
			$aTitlePart = array();
			$aTitlePart[] = DenifLink::createAction('#', 'cat-details')->html();
			if ($idKey) {
				$aAttrTitle += attr::id($key->getAnchor());
				$key->iObjectLinksFlags = ($aRow) ? 0 : objectLinksDelete;
				$aTitleLink = $this->getObjectLinks($key, $key->getAnchor(), $key->iObjectLinksFlags);
				append($aTitleLink, $this->getPropertyFlagsIcon($key));
				$aTitlePart[] = concat($aTitleLink);
				$sTitle = $key->getLinkTitle();
				$sTitleSuffix = $key->linkOut($this, true);
			}else{
				$sTitle = '(Sans ' . $type->sAlone . ')';
				$sTitleSuffix = '';
			}
			$sTitle = html::text($sTitle);
			$aTitlePart[] = html::link('#', $sTitle, $aAttrTitle) . $sTitleSuffix;
			
			if ($idKey && $idKey == $this->id)
				$aAttrWhole = attr::idClassName('id', 'cat-on');
			else if ($this->id == 0)
				$aAttrWhole = attr::className('cat-on');
			else
				$aAttrWhole = attr::className('cat-off');
			
			if ($aRow)
				$sList = self::getTable($aRow, $aColWidth);
			else if ($this->isEditing() || $bAdmin)
				$sList = html::par('Aucune ressource pour ' . $type->sLeLa, attr::className('cat-contents'));
			else
				continue;
			
			$sKeyList .= html::div(
				concat($aTitlePart) .
				$sList,
				$aAttrWhole);
		}
		
		return
			$this->getCreatePropertyResourceLinkBar($iPropertyKey) .
			html::div(
				html::link('javascript:catAll("on")', 'Ouvrir tout') . '  ' .
				html::link('javascript:catAll("off")', 'Fermer tout'),
				attr::className('cat-jsbar')) .
			html::div($sKeyList, attr::id('cats'));
	}
}

?>