<?php defined('KERNEL_INCLUDED') or die('kernel should be included first');
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

//-----------------------------------------------------------------
/// @file
/// @brief #PageDenif: DENIF pages base class
//-----------------------------------------------------------------


/// @brief Flags for PageDenif::getObjectLinks().
defineFlags('objectLinks', 'Details;Delete;Move;Zip');

/// @brief Flags for PageDenif::getSelfTarget().
defineEnum('target', 'Session;Params');


//-----------------------------------------------------------------
/// DENIF pages base class
//-----------------------------------------------------------------

abstract class PageDenif extends Page
{
	public $sessionManager;
	
	private $sError = null, $bLoginForm = false;
	
	
	protected function getTitle() { return sInstanceAcronym; }
	protected function getAuthor() { return sInstanceAuthor; }
	protected function getCSS() { return sRootRel . 'style.css'; }
	
	
	protected abstract function contents();
	
	
	protected function initialize()
	{
		parent::initialize();
		
		$this->bLoginForm = $_REQUEST->getSafe('bLoginForm');
		
		$this->sessionManager = new SessionManager();
	}
	
	
	protected function head()
	{
		$s = parent::head();
		
		if ($this->bLoginForm || $this->sError)
			$s .= html::meta('robots', 'noindex,nofollow');
		
		return $s;
	}
	
	protected function scripts()
	{
		$s = parent::scripts();
		
		$sOnLoad = '';
		
		if (!bCanSendEmail)
			$sOnLoad .= 'unobfuscateMail(id("denif-mail"));';
		
		if ($this->bLoginForm)
			$sOnLoad .= 'id("flogin").sLogin.focus();';
		
		if ($sOnLoad) {
			$s .= html::jscriptLink('global.js') .
				html::jscriptInline('catchLoad(function() {' . $sOnLoad . '});');
		}
		
		return $s;
	}
	
	
	protected function body()
	{
		$s = '';
		
		$sTop = '';
		
		// Login box
		$sLogin = $this->sessionManager->getLogin();
		$linkLogin = ($sLogin)
			? new DenifLink($this->getSelfTarget(targetParams, array('bLogout' => true)),
				'Dconnexion', 'withicon logout', 'Connect sous ' . html::text($sLogin))
			: new DenifLink($this->getSelfTarget(targetParams, array('bLoginForm' => true)),
				'Connexion', 'withicon login');
		$sTop .=
			html::div(
				$this->linkTarget('index.php', 'Accueil', 'withicon home') .
				$this->linkInside($linkLogin),
				attr::id('boxlogin'));
		
		// Main title
		$sTitle = $this->linkTarget('index.php',
			html::span(sInstanceAcronym . '  ', attr::className('acronymtitle')) .
			html::span(sInstanceTitle, attr::className('fulltitle')), '', 'Page daccueil');
		if (bLocal)
			$sTitle .= ' ' . html::span('VERSION LOCALE',
				attr::style('font-weight: bold; font-size: x-large; color: white; background: red; border: 2px solid red; padding: 0 0.2em'));
		
		$sTop .= html::title(1, $sTitle);
		
		// Navigation bar
		$sTop .= html::div($this->getHeaderBox(), attr::id('headerbox'));
		
		$s .= $sTop;
		
		// Login form or page contents
		$this->bLoginForm =
			!$this->isLogged() &&
			($this->bLoginForm || $_REQUEST->getSafe('bSetEdit'));
		if ($this->bLoginForm)
			$sContents = $this->getLoginForm();
		else{
			if (!$this->sError) {
				try{
					$this->processMove();
					$sContents = $this->contents();
				}catch(PageException $e) {
				}
			}
		}
		if ($this->sError)
			$this->bLoginForm = false;
		
		// Errors or page contents
		$s .= (!$this->bLoginForm && $this->sError)
			? $this->getError()
			: $sContents;
		
		$s .= html::div($this->getFooterBox(), attr::id('footerbox'));
		
		// Debug statistics
		if ($this->isAdmin()) {
			if (bLocal && false) {
				$s .= html::div('Requtes SQL&nbsp;: ' . sql::$nbQuery, attr::className('generation-stats'));
				if (sql::$aQuery)
					$s .= html::pre(concat(sql::$aQuery, CRLF));
			}
			$s .= html::div(
				'Gnr en&nbsp;: ' . str_replace('.', ',', elapsedTime()) .
				'  Taille&nbsp;: ' . (strlen($s) >> 10) . '&nbsp;Ko',
				attr::className('generation-stats'));
		}
		
		// Google analytics statistics
		if (defined('sGoogleAnalyticsId') && !$this->isAdmin())
			if (sGoogleAnalyticsId)
				$s .= CRLF . $this->getGoogleAnalytics(sGoogleAnalyticsId);
		
		return $s;
	}
	
	/// Raise a fatal error if the user is not logged or is not an admin.
	protected function needAdmin()
	{
		if (!$this->isAdmin())
			$this->fatalError('Page rserve aux administrateurs.');
	}
	
	protected function fatalError($sMessage)
	{
		$this->sError = $sMessage;
		throw new PageException;
	}
	
	protected function getError()
	{
		return ($this->sError) ? html::div($this->sError, attr::className('error')) : null;
	}
	
	
	//-----------------------------------------------------------------
	/// @name Parameters
	/// @{
	//-----------------------------------------------------------------
	
	/// @brief \c ';' separated string specifying the parameters
	/// that loadParameters() should load from \c $_REQUEST
	/// to store them as member in the object.
	/// The parameters should be \c public properties of the class.
	protected function getParameters() { return ''; }
	
	public function getParametersValue()
	{
		$a = array();
		foreach (getList($this->getParameters()) as $sName)
			$a[$sName] = $this->$sName;
		return $a;
	}
	
	/// @brief Load the values specified by getParameters()
	/// from \c $_REQUEST and store them as member in the object.
	/// The parameters should be \c public properties of the class.
	protected function loadParameters()
	{
		$this->loadRequestParams($this->getParameters());
	}
	
	/// @}
	
	
	//-----------------------------------------------------------------
	/// @name Mode
	/// @{
	//-----------------------------------------------------------------
	
	public function isLogged()    { return $this->sessionManager->isLogged(); }
	public function isModerator() { return $this->sessionManager->isModerator(); }
	public function isAdmin()     { return $this->sessionManager->isAdmin(); }
	public function isEditing()   { return $this->sessionManager->isEditing(); }
	
	public function wantEditing($bErrorIfImpossible)
	{
		if ($this->sessionManager->wantEditing())
			return true;
		
		if ($bErrorIfImpossible)
			$this->fatalError('Vous devez tre en mode dition.');
		
		$this->bLoginForm = $_REQUEST->bSetEdit = true;
		return false;
	}
	
	/// @}
	
	
	//-----------------------------------------------------------------
	/// @name Links
	/// @{
	//-----------------------------------------------------------------
	
	public function targetAddSessionParams(target $target)
	{
		$this->sessionManager->targetAddSessionParams($target);
	}
	
	public static function targetGetCleanURL(target $target)
	{
		$target = clone $target;
		$target->removeParams('sSession;bEdit');
		return $target->url();
	}
	
	function getSelfTarget($iLevel, array $aParam = array(), $sAnchor = '')
	{
		$target = new target(sSelfName, array(), $sAnchor);
		if ($iLevel >= targetSession) {
			$this->targetAddSessionParams($target);
			if ($iLevel >= targetParams)
				$target->addParams($this->getParametersValue());
		}
		$target->addParams($aParam);
		return $target;
	}
	
	function linkOut($sTarget, $sLabel, $sClassName = '', $sToolTip = '')
	{
		return ($sTarget)
			? html::link($sTarget, $sLabel, attr::className($sClassName) + attr::title($sToolTip))
			: $sLabel;
	}
	
	function linkInside(DenifLink $link)
	{
		if ($link->target->url() != '#')
			$this->targetAddSessionParams($link->target);
		return $link->html();
	}
	
	function mapLinkInside(array $aLink)
	{
		return map(array($this, 'linkInside'), $aLink);
	}
	
	function linkTarget($target, $sLabel, $sClassName = '', $sToolTip = '')
	{
		return $this->linkInside(new DenifLink($target, $sLabel, $sClassName, $sToolTip));
	}
	
	function targetFromURL($sURL)
	{
		$target = target::fromURL($sURL);
		$this->targetAddSessionParams($target);
		return $target;
	}
	
	/// @}
	
	
	//-----------------------------------------------------------------
	/// @name HTML writing
	/// @{
	//-----------------------------------------------------------------
	
	private $bFirstTitle = true;
	
	public function putTitle($sTitle, $sAnchor = '', $iLevel = 0)
	{
		$aAttr = attr::id($sAnchor);
		if ($this->bFirstTitle) {
			$this->bFirstTitle = false;
			$aAttr += attr::className('first');
		}
		
		return html::title(2 + $iLevel, $sTitle, $aAttr);
	}
	
	public function linkBar($sTitle, array $aElem)
	{
		if (!$aElem)
			return '';
		$s = '';
		if ($sTitle)
			$s .= html::b($sTitle . '&nbsp;: ');
		
		$sFirst = array_shift($aElem);
		$s .= html::span($sFirst, attr::className('first'));
		foreach ($aElem as $sElem)
			$s .= ' ' . html::span($sElem, attr::className('next'));
		
		return html::div($s, attr::className('linkbar'));
	}
	
	public function editBar(array $aElem)
	{
		return $this->linkBar(null, $aElem);
	}
	
	
	//-----------------------------------------------------------------
	/// @name Navigation bar
	/// @{
	//-----------------------------------------------------------------
	
	private function getBarEdit()
	{
		$bEditing = $this->isEditing();
		
		$aLink = array();
		
		$targetSelf = $this->getSelfTarget(targetParams);
		$aLink[] = $this->linkTarget($targetSelf, 'Recharger la page');
		
		if ($this->isAdmin()) {
			$aParam = array('sReturnURL' => $this->targetGetCleanURL($targetSelf));
			$idSource = $this->getCurrentNodeID();
			if ($idSource)
				$aParam['idSource'] = $idSource;
			
			$aLink[] = $this->linkTarget(new target('nodes.php',
				array('bNotFinalOnly' => true, 'bForceChildren' => true)), 'Ressources proposes');
			$aLink[] = $this->linkTarget(new target('want_download.php',
				$aParam + array('bPingDownload' => 1)), 'Tlcharger');
			$aLink[] = $this->linkTarget(new target('want_download.php', $aParam), 'Tlchargement');
			
			if ($bEditing)
				$aLink[] = $this->linkTarget(new target('move_node.php', $aParam), 'Dplacer nud');
		}
		
		$aLink[] = $this->linkTarget(
			$this->getSelfTarget(targetParams, array('bSetEdit' => !$bEditing)),
			'Mode ' . (($bEditing) ? 'navigation' : 'dition'));
		
		return $aLink;
	}
	
	protected function hasSearchForm() { return true; }
	
	/// @brief Return the ID of the currently displayed object.
	/// Will be used as default source by the 'move node' link.
	/// Return \c 0 if there is no current node.
	protected function getCurrentNodeID() { return 0; }
	
	private function getBarSearch(array& $aFormAttr)
	{
		$a = array();
		
		$aLink = array(
			'index_themes.php'   => 'Thmes',
			'index_cursus.php'   => 'Cursus',
			'index_teachers.php' => 'Enseignants',
		);
		foreach ($aLink as $sURL => $sTitle) {
			if (sSelfName == $sURL)
				$sTitle = html::span($sTitle, attr::className('current'));
			$a[] = $this->linkTarget($sURL, $sTitle);
		}
		
		if ($this->hasSearchForm()){
			$sForm = implode(' ', array(
				form::text('q', '', 30, maxLengthSearch),
				form::submit('sa', 'Rechercher'),
			));
			
			$sForm .= $this->getGoogleSearchCode($aFormAttr);
			
			// The <form> tag is block-level, it needs to be written
			// around the whole form: see getHeaderBox() for it.
			$a[] = html::span($sForm, attr::className('search'));
		}
		
		return $a;
	}
	
	protected function getGoogleSearchCode(array& $aFormAttr)
	{
		return (bGoogleSearch)
			? $this->getGoogleCSEFormHidden($aFormAttr, sGoogleCSEId)
			: '';
	}
	
	private function getBarAdmin()
	{
		$aLink = array();
		
		$aLink[] = $this->linkTarget('nodes.php', 'Tout sur une page');
		
		if ($this->isAdmin()) {
			$aLink[] = $this->linkTarget('maintenance.php', 'Maintenance');
			$aLink[] = $this->linkOut('phpMyAdmin/', 'phpMyAdmin');
		}
		
		$aLink[] = $this->linkTarget(
			$this->getSelfTarget(targetParams, array('bSimulateGuest' => true)),
			'Simuler non admin');
		
		return $aLink;
	}
	
	private function getHeaderBox()
	{
		$s = '';
		
		if ($this->isAdmin())
			$s .= $this->linkBar(null, $this->getBarAdmin());
		
		if ($this->isLogged())
			$s .= $this->editBar($this->getBarEdit());
		
		$aFormAttr = array();
		$s .= $this->editBar($this->getBarSearch($aFormAttr));
		
		// W3C says that <form> is block-level:
		// put the <form> around the whole header box.
		if ($this->hasSearchForm()) {
			$target = new target('search.php');
			$this->targetAddSessionParams($target);
			$s = $target->form(form::METHOD_GET, $s, $aFormAttr);
		}
		
		return $s;
	}
	
	private function getFooterBox()
	{
		$aMapLink = array(
			new DenifLink('index.php',   'Accueil'),
			new DenifLink('sitemap.php', 'Plan du site'),
		);
		if (bCanSendEmail)
			$aMapLink = array_merge($aMapLink, array(
				new DenifLink('contact.php',            'Contacter un administrateur'),
				new DenifLink('contact.php?bPropose=1', 'Proposer des documents'),
			));
		$aItem = $this->mapLinkInside($aMapLink);
		
		if (!bCanSendEmail)
			$aItem[] = 'Contact et proposition de documents&nbsp;: ' .
				html::kbd(obfuscateMail(sContactEmail), attr::id('denif-mail'));
		
		$s = $this->editBar($aItem);
		$s .= html::div(sDocumentsCopyrightNotice, attr::className('copyrightnotice'));
		
		$sLicense = (bPrimaryInstance) ? sPrimaryInstanceLicense : sSecondaryInstanceLicense;
		$s .= html::div($sLicense, attr::className('primaryinstancenotice'));
		
		return $s;
	}
	
	
	//-----------------------------------------------------------------
	/// @name Contents
	/// @{
	//-----------------------------------------------------------------
	
	private function getLoginForm()
	{
		$form = new FormManager($this);
		$form->pushElement(form::label('sLogin', 'Login'),
			form::text('IDsLogin', $_POST->getSafe('sLogin'), 12));
		$form->pushElement(form::label('sPassword', 'Mot de passe'),
			form::password('IDsPassword', '', 12));
		$form->pushElement('', form::checkboxLabel('bCookie', 'Se [s]ouvenir de moi (cookie)'));
		$form->pushElement('', form::submit('bLogin', 'Entrer'));
		
		return
			$this->putTitle($this->sessionManager->getLogoutMessage()) .
			$this->getSelfTarget(targetParams, array(
				'bLoginForm' => true,
				'bSetEdit'   => $_REQUEST->getSafe('bSetEdit'),
			))->form(form::METHOD_POST, $form->html(),
			attr::id('flogin'));
	}
	
	/// @}
	
	
	//-----------------------------------------------------------------
	/// @name Edition links
	/// @{
	//-----------------------------------------------------------------
	
	public function linkEdit(DenifLink $link)
	{
		return $this->linkInside($link);
	}
	
	public function mapLinkEdit(array $aLink)
	{
		return map(array($this, 'linkEdit'), $aLink);
	}
	
	public function linkEditBar(array $aLink)
	{
		return $this->editBar($this->mapLinkEdit($aLink));
	}
	
	
	/// @brief Create the links that can manipulate an object.
	/// If there is at least one link, a \c '&nbsp' element is appended.
	///
	/// Flags meaning:
	/// - If not \c objectLinksDelete, the \c 'delete' link is disabled but visible.
	/// - If not \c objectLinksMove, the \c 'move' links are completely hidden.
	/// - If \c objectLinksMove and <tt>$iOrderLast \< 0</tt>, the \c 'move' links are disabled but visible.
	/// @return an array containing one element per link for the object.
	public function getObjectLinks(ObjectBase $obj, $sAnchor, $iFlags, $iOrderLast = -1, $targetDelete = null)
	{
		$aLink = array();
		$aFlagIcon = array();
		
		if ($iFlags & objectLinksDetails)
			$aLink[] = $obj->linkDetails($this);
		
		if ($iFlags & objectLinksZip)
			$aLink[] = $obj->linkZip($this);
		
		if ($this->isEditing()) {
			$bMove = (bool)($iFlags & objectLinksMove);
			
			$sActions = ($bMove) ? ';up;down' : '';
			foreach (getList('edit;delete' . $sActions) as $sAction)
				$aLink[$sAction] = null;
			
			$target = $this->getSelfTarget(targetParams, array(), $sAnchor);
			
			if (($iFlags & objectLinksDelete) && $this->sessionManager->hasRightDelete($obj)) {
				if (!$targetDelete)
					$targetDelete = $this->getSelfTarget(targetParams);
				$aLink['delete'] = getLinkDelete($obj, $targetDelete, clone $target);
			}
			
			if ($this->sessionManager->hasRightModify($obj)) {
				$aLink['edit'] = getLinkEdit($obj, clone $target);
				if ($bMove && $iOrderLast >= 0 && ($obj instanceof node)) {
					$aLink['up']   = getLinkMove($obj, -1, $iOrderLast, clone $target);
					$aLink['down'] = getLinkMove($obj, +1, $iOrderLast, clone $target);
				}
			}
			
			foreach ($aLink as $sAction => &$link)
				if ($link === null)
					$link = getLinkDisabled($sAction);
			
			if ($obj->getType() == nodeDocGroup)
				$aLink[] = getLinkEditDocs($obj, clone $target);
		}
		
		$a = $this->mapLinkInside($aLink);
		if ($a)
			$a[] = html::nbsp();
		if ($aFlagIcon) {
			append($a, $aFlagIcon);
			$a[] = html::nbsp();
		}
		return $a;
	}
	
	/// @brief Create the icons reflecting the flags of a node.
	/// Return an array containing one element per icon.
	public function getNodeFlagsIcon(node $node)
	{
		$aFlagIcon = array();
		
		if ($this->isEditing() && $node->aFlagInherit[nodeFlagFinal] != finalTemp) {
			foreach ($node->parent->getNodeFlags() as $nodeFlag) {
				$sPHPName = $nodeFlag->sPHPName;
				$iValue = $node->$sPHPName;
				$aFlagIcon[] = getFlagIcon($nodeFlag->aImage[$iValue], $nodeFlag->aValue[$iValue]);
			}
			
			$aFlagIcon[] = html::nbsp();
		}
		
		return $aFlagIcon;
	}
	
	/// @brief Create the icons reflecting the flags of a property.
	/// Return an array containing one element per icon.
	public function getPropertyFlagsIcon(property $property)
	{
		$aFlagIcon = array();
		
		if ($this->isEditing()) {
			$bValidated = $property->bValidated;
			
			$aFlagIcon[] = getFlagIcon('final ' . (($bValidated) ? 'yes' : 'no'),
				($bValidated) ? 'Valid' : 'Non valid');
			
			$aFlagIcon[] = html::nbsp();
		}
		
		return $aFlagIcon;
	}
	
	public function getCreatePropertyResourceLinkBar($iProperty = 0)
	{
		$aLink = array();
		if ($this->isEditing()) {
			$targetSelf = $this->getSelfTarget(targetSession);
			if ($iProperty && $this->sessionManager->hasRightCreate($iProperty))
				$aLink[] = getLinkCreate($iProperty, clone $targetSelf);
			if ($this->sessionManager->hasRightCreate(nodeResource))
				$aLink[] = getLinkCreate(nodeResource, clone $targetSelf);
		}
		return $this->linkEditBar($aLink);
	}
	
	
	public function processMove()
	{
		extract($_REQUEST->getSafeList('iMove;idMove;iMoveFrom'));
		if (!$iMove)
			return;
		
		$node = getNode($idMove);
		$node->loadAllParents(true);
		if (!$node || $node->iOrder != $iMoveFrom ||
		    !$this->sessionManager->hasRightModify($node))
			return;
		
		$aWhere = array('iNode' => $node->iNode);
		if ($node->idParent)
			$aWhere['idParent'] = $node->idParent;
		
		$sTable = node::$table->sName;
		sql::lockTables(array($sTable));
		
		$iOrderOther = $node->iOrder + (($iMove > 0) ? +1 : -1);
		sql::update($sTable,
			array('iOrder' => $node->iOrder),
			sql::where($aWhere + array('iOrder' => $iOrderOther)));
		if (sql::getAffectedCount() > 0)
			sql::update($sTable,
				array('iOrder' => $iOrderOther),
				sql::where($aWhere + array('id' => $node->id)));
		
		sql::unlockTables();
	}
	
	/// @}
}


function getFlagIcon($sAction, $sToolTip = '')
{
	return html::span(html::nbsp() . html::nbsp(),
		attr::title($sToolTip) +
		attr::className('icon flag ' . $sAction));
}


?>